@@ -2,11 +2,11 @@ use 5.008;
 use strict;
 use warnings;
 package BenchmarkAnything::Storage::Frontend::Lib;
-BEGIN {
-  $BenchmarkAnything::Storage::Frontend::Lib::AUTHORITY = 'cpan:SCHWIGON';
-}
+# git description: v0.013-5-g3659794
+
+our $AUTHORITY = 'cpan:SCHWIGON';
 # ABSTRACT: Basic functions to access a BenchmarkAnything store
-$BenchmarkAnything::Storage::Frontend::Lib::VERSION = '0.013';
+$BenchmarkAnything::Storage::Frontend::Lib::VERSION = '0.014';
 use Scalar::Util 'reftype';
 
 
@@ -294,6 +294,67 @@ sub createdb
 }
 
 
+sub _default_additional_keys
+{
+        my ($self) = @_;
+
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
+        {
+                return { $self->{backend}->default_columns };
+        }
+        else
+        {
+                # Hardcoded from BenchmarkAnything::Storage::Backend::SQL::Query::common,
+                # as it is a backend-special and internal thing anyway.
+                return {
+                    'NAME'      => 'b.bench',
+                    'UNIT'      => 'bu.bench_unit',
+                    'VALUE'     => 'bv.bench_value',
+                    'VALUE_ID'  => 'bv.bench_value_id',
+                    'CREATED'   => 'bv.created_at',
+                };
+        }
+}
+
+
+
+sub _get_benchmark_operators
+{
+        my ($self) = @_;
+
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
+        {
+                return [ $self->{backend}->benchmark_operators ];
+        }
+        else
+        {
+                # Hardcoded from BenchmarkAnything::Storage::Backend::SQL::Query::common,
+                # as it is a backend-special and internal thing anyway.
+                return [ '=', '!=', 'like', 'not like', '<', '>', '<=', '>=' ];
+        }
+}
+
+
+
+sub _get_additional_key_id
+{
+        my ($self, $key_name) = @_;
+
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
+        {
+                return $self->{backend}->_get_additional_key_id($key_name);
+        }
+        else
+        {
+                die "benchmarkanything: no backend '$backend' allowed here, available backends are: 'local'.\n";
+        }
+}
+
+
+
 sub init_workdir
 {
         my ($self) = @_;
@@ -535,6 +596,35 @@ sub listnames
 }
 
 
+sub listkeys
+{
+        my ($self, $pattern) = @_;
+
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
+        {
+                return $self->{backend}->list_additional_keys(defined($pattern) ? ($pattern) : ());
+        }
+        elsif ($backend eq 'http')
+        {
+                my $ua  = $self->_get_user_agent;
+                my $url = $self->_get_base_url."/api/v1/listkeys";
+
+                my $res = $ua->get($url)->res;
+                die "benchmarkanything: ".$res->error->{message}." ($url)\n" if $res->error;
+
+                my $result = $res->json;
+
+                # output
+                return $result;
+        }
+        else
+        {
+                die "benchmarkanything: no backend '$backend', available backends are: 'http', 'local'.\n";
+        }
+}
+
+
 sub gc
 {
         my ($self) = @_;
@@ -770,6 +860,24 @@ operation it verifies an "are you sure" test, by comparing the
 parameter C<really> against the DSN from the config, or if that
 doesn't match, asking interactively on STDIN.
 
+=head2 _default_additional_keys
+
+Internal method. Specific to SQL backend.
+
+Return default columns that are part of each BenchmarkAnything data point.
+
+=head2 _get_benchmark_operators
+
+Internal method. Specific to SQL backend.
+
+Return the allowed operators of the BenchmarkAnything query API.
+
+=head2 _get_additional_key_id
+
+Internal method. Specific to SQL backend.
+
+Returns id of the additional key.
+
 =head2 init_workdir
 
 Initializes a work directory C<~/.benchmarkanything/> with config
@@ -792,6 +900,12 @@ Returns an array ref with all metric NAMEs. Optionally allows to
 restrict the search by a SQL LIKE search pattern, allowing C<%> as
 wildcard.
 
+=head2 listkeys ($pattern)
+
+Returns an array ref with all additional key names that are used for
+metrics. Optionally allows to restrict the search by a SQL LIKE search
+pattern, allowing C<%> as wildcard.
+
 =head2 gc()
 
 Run garbage collector. This cleans up potential garbage that might
@@ -821,7 +935,7 @@ Steffen Schwigon <ss5@renormalist.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by Steffen Schwigon.
+This software is copyright (c) 2016 by Steffen Schwigon.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
